% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{pairdiff_mean}
\alias{pairdiff_mean}
\title{Distance matrix between columns of a matrix using their mean differences}
\usage{
pairdiff_mean(x, cols = NULL, min_value_count = 3L, na_mode = "check")
}
\arguments{
\item{x}{a numeric matrix that needs to be normalized. This variable is changed by reference! i.e. after this function, the original variable is updated}

\item{cols}{optionally, provide an integer vector with column indices (in \code{x}) that should be used (these should be 1-based indices as per usual in R). Or set to \code{NULL} or \code{integer()} to use all columns}

\item{min_value_count}{the minimum number of values overlapping between a pair of columns (if there are fewer overlapping values, respective rescaling is not computed)}

\item{na_mode}{string value that indicated how should we should deal with NA values (default). "check" = test if NA values are present and if so, remove these. "present" = you already know NA values are present so we can skip the check for NA values for efficiency. "unchecked" = you guarantee that there are no NA values in input, thus we'll use the fastest code paths that skip over any NA checks downstream; ONLY select this if you are sure there are no NA/NaN/Inf/-Inf values !}
}
\value{
a N x N numeric matrix (where N is number of column in input \code{x}) representing the mean difference between each column
}
\description{
Pairwise differences between all columns in a matrix
}
