% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{vector_trimmedmean}
\alias{vector_trimmedmean}
\title{Compute trimmed mean value of a vector, with optional filtering for N datapoints}
\usage{
vector_trimmedmean(x, min_value_count = 1L, trim = 0.2, na_mode = "check")
}
\arguments{
\item{x}{numeric input vector, may contain non-finite values (removed if \code{na_mode} is left to default)}

\item{min_value_count}{the minimum number of values overlapping between a pair of columns (if there are fewer overlapping values, respective rescaling is not computed)}

\item{trim}{amount of trim to apply to both the lower- and upper-parts of a vector before computing the mean. 0 indicates no trim, 0.5 indicates 100\% trim (i.e. 50\% of data on both sides) so that value is out of bounds. Typically set to 0.1-0.3}

\item{na_mode}{string value that indicated how should we should deal with NA values (default). "check" = test if NA values are present and if so, remove these. "present" = you already know NA values are present so we can skip the check for NA values for efficiency. "unchecked" = you guarantee that there are no NA values in input, thus we'll use the fastest code paths that skip over any NA checks downstream; ONLY select this if you are sure there are no NA/NaN/Inf/-Inf values !}
}
\value{
a single numeric value representing the trimmed mean
}
\description{
Compute measure of central tendency using efficient C++ code
}
