% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pairscale.R
\docType{package}
\name{pairscale}
\alias{pairscale-package}
\alias{pairscale}
\title{pairscale package declaration}
\description{
Normalization of numerical matrices by minimizing the mean/median/mode difference between all column pairs.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/ftwkoopmans/pairscale/}
}

}
\author{
\strong{Maintainer}: Frank Koopmans \email{ftwkoopmans@gmail.com} (\href{https://orcid.org/0000-0002-4973-5732}{ORCID})

}
\keyword{internal}
